import sys
import contextlib

import numpy as np
import scipy as sp

from values import estimate_shapley, estimate_sample_count


def parse_args(argv):
    assert len(argv) == 3, "Usage: main.py delta eps"

    delta = float(argv[1])
    epsilon = float(argv[2])

    return delta, epsilon


def parse_flow_game(io):
    """
    Parses a flow-network task. Vertex ID 0 is the source and ID 1 is the sink.
    Returns:
        - directed incidence matrix B[v, e] -> {-1, 0, 1}
        - edge--capacity (upper-bound) array C[e] -> float32
        - agent--edge (ownership) bitmap A[a, e] -> {0, 1}.
    """

    np_f32 = np.float32
    np_int = np.int32

    (nv, ne, na) = np_int(io.readline().split())
    (S, T, C, A) = np.array([line.split() for line in io.readlines()]).T
    (S, T, C, A) = np_int(S), np_int(T), np_f32(C), np_int(A)

    Eids = range(ne)
    ones = np.ones(ne)

    Ss = sp.sparse.csr_matrix((ones, (S, Eids)), shape=(nv, ne), dtype=int)
    Ts = sp.sparse.csr_matrix((ones, (T, Eids)), shape=(nv, ne), dtype=int)
    As = sp.sparse.csr_matrix((ones, (A, Eids)), shape=(na, ne), dtype=int)

    return (Ss - Ts), C, As


def print_result(io, allocations):
    np.savetxt(io, allocations, fmt="%.3f", newline=" ")
    print(file=io)


if __name__ == "__main__":
    delta, epsilon = parse_args(sys.argv)
    B, C, A = parse_flow_game(sys.stdin)

    with contextlib.redirect_stdout(sys.stderr):
        sample_count = estimate_sample_count(B, C, A, delta, epsilon)
        estimate = estimate_shapley(B, C, A, sample_count=sample_count)

    print_result(sys.stdout, estimate)
